import {html} from './deps.js';

import {loadView} from "./view/control/loadView.js";
import BaseElement from "./BaseElement.js";

//We load upfront all modules for now. Consider lazy-loading/code splitting later.
import About from './about/boundary/About.js';
import Account from './account/boundary/Account.js';
import Audit from './audit/boundary/Audit.js';
import Contact from './contact/boundary/Contact.js';
import Financials from './financials/boundary/Financials.js';
import Home from './home/boundary/Home.js';
import Incidents from './incidents/boundary/Incidents.js';
import Insurance from './insurance/boundary/Insurance.js';
import LabTests from './labtests/boundary/LabTests.js';
import Messages from './messages/boundary/Messages.js';
import Patients from './patients/boundary/Patients.js';
import Programs from './programs/boundary/Programs.js';
import Residencies from './residencies/boundary/Residencies.js';
import User from './user/boundary/User.js';


export default class App extends BaseElement {

    static #routes = {
        'default': _ => new Home(),
        'home': _ => new Home(),
        'user': _ => new User(),
        'messages': _ => new Messages(),
        'account': _ => new Account(),
        'patients': _ => new Patients(),
        'residencies': _ => new Residencies(),
        'financials': _ => new Financials(),
        'insurance': _ => new Insurance(),
        'labtests': _ => new LabTests(),
        'programs': _ => new Programs(),
        'incidents': _ => new Incidents(),
        'audit': _ => new Audit(),
        'contact': _ => new Contact(),
        'about': _ => new About()
    };

    #startButton;
    #sideNavigation;
    #shellBar;
    #main;
    #domContentLoadedListener;

    static routeToView(link) {
        window.location.hash = link;
    }

    connectedCallback() {
        super.connectedCallback();
        //Initialize shell and side bars
        this.#main = document.querySelector("main");
        this.#shellBar = document.querySelector("ui5-shellbar");
        this.#sideNavigation = document.querySelector("ui5-side-navigation");
        this.#startButton = document.querySelector("#startButton");
        this.#startButton.addEventListener("click", _ => this.#sideNavigation.collapsed = !this.#sideNavigation.collapsed);
        this.#sideNavigation
            .querySelectorAll('ui5-side-navigation-item')
            .forEach(navigationItem => navigationItem.addEventListener('click', e => App.routeToView(e.target.id)));

        window.onhashchange = _ => this.#onAddressBarChanged();
        this.#domContentLoadedListener = () => this.#onAddressBarChanged();
        document.addEventListener('DOMContentLoaded', this.#domContentLoadedListener);
        App.routeToView('home')
    }

    disconnectedCallback() {
        super.disconnectedCallback();
        document.removeEventListener('DOMContentLoaded', this.#domContentLoadedListener);
    }

    extractSlice(reduxState) {
        return reduxState.user;
    }

    onStateUpdate() {
        const user = this.slice;
        if (user.jwt && user.loading === false) {
            console.log("User has been authenticated!", user);
        }
    }

    static createViewFor = link => {
        const factoryMethod = App.#routes[link];
        return !factoryMethod ? App.#routes['default']() : factoryMethod();
    }

    #onAddressBarChanged() {
        const {location: {hash}} = window;
        const link = hash.substring(1);
        loadView(link);
    }

    view() {
        return html`
            <style>
                recres-navigation {
                    height: 100vh;
                    display: grid;
                    grid-template-rows: auto 1fr;
                    grid-template-columns: auto 1fr;
                    grid-template-areas:
                        "shellbar shellbar"
                        "sidebar view";
                }

                ui5-shellbar {
                    grid-area: shellbar;
                }

                ui5-side-navigation {
                    grid-area: sidebar;
                }

                recres-view {
                    grid-area: view;
                    background-color: lightgoldenrodyellow;
                }
            </style>

            <ui5-shellbar id="shellBar" primary-title="Rec Res" show-notifications>
                <ui5-button icon="menu" id="startButton" slot="startButton"></ui5-button>
                <!--            <img id="logo" slot="logo" src="./images/logo.svg"/>-->
                <ui5-input placeholer="Enter service..." slot="searchField"></ui5-input>
                <!--            <ui5-avatar slot="profile" >John S.s</ui5-avatar>-->
                <ui5-avatar icon="customer" slot="profile"></ui5-avatar>
                <!--            <ui5-badge slot="profile">JS</ui5-badge>-->
            </ui5-shellbar>

            <ui5-side-navigation>

                <ui5-side-navigation-item icon="home" id="home" text="Home"></ui5-side-navigation-item>

                <ui5-side-navigation-item icon="key-user-settings" id="user" text="User">
                    <ui5-side-navigation-sub-item text="Update"></ui5-side-navigation-sub-item>
                    <ui5-side-navigation-sub-item text="Security"></ui5-side-navigation-sub-item>
                    <ui5-side-navigation-sub-item text="Activity"></ui5-side-navigation-sub-item>
                    <ui5-side-navigation-sub-item text="Sign out"></ui5-side-navigation-sub-item>
                    <ui5-side-navigation-sub-item text="Delete"></ui5-side-navigation-sub-item>
                </ui5-side-navigation-item>

                <ui5-side-navigation-item icon="bell" id="messages" text="Messages"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="company-view" id="account" text="Account"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="customer" id="patients" text="Patients"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="building" id="residencies"
                                          text="Residencies"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="money-bills" id="financials"
                                          text="Financials"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="insurance-life" id="insurance"
                                          text="Insurance"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="lab" id="labtests" text="Lab tests"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="product" id="programs" text="Programs"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="incident" id="incidents" text="Incidents"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="log" id="audit" text="Audit"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="email" id="contact" slot="fixedItems"
                                          text="Contact Us"></ui5-side-navigation-item>
                <ui5-side-navigation-item icon="message-information" id="about" slot="fixedItems"
                                          text="About"></ui5-side-navigation-item>

            </ui5-side-navigation>

            <recres-view></recres-view>
        `;
    }

}

customElements.define('recres-navigation', App);